/* Copyright 2000-2003 Hiroshi Watabe*/
/* histgram ROI */
/* $Id: hist_proc.c,v f4d449a26330 2010/04/14 21:14:30 watabe $ */
#include "gpetview.h"

unsigned long hist[256];
unsigned char hist_on_flag = 0;
gfloat *fhist = NULL;
GtkCurve *hist_curve;
int hist_imax,hist_ymax;

void save_hist() /* save histgram */
{
  FILE *fp;
  GtkWidget *hist_entry;
  int i;

  hist_entry = lookup_widget(hist_window,"hist_entry");

  fp = fopen(gtk_entry_get_text(GTK_ENTRY(hist_entry)),"wt");
  for(i=0;i<256;i++){
    fprintf(fp,"%g %ld\n",(float)i*(img->scalefactor),hist[i]);
  }
  fclose(fp);
}
void hist_out()
{
  int i;

  gtk_widget_show(hist_window);
  hist_curve = (GtkCurve *)lookup_widget(hist_window,"hist_curve");

  for(i=255,hist_imax=255;i>0;i--){
    if(hist[i]>0) break;
  }
  hist_imax = i;
  if(fhist==NULL){
    fhist = (gfloat *)g_malloc(hist_imax*sizeof(gfloat));
  }
  for(i=0,hist_ymax=0;i<hist_imax;i++){
    if(hist_ymax<hist[i]) hist_ymax = hist[i];
    fhist[i] = (float)hist[i];
  }
  gtk_curve_set_range(hist_curve,0,hist_imax,0,hist_ymax);
  gtk_curve_set_vector(hist_curve,hist_imax,fhist);
  gtk_curve_set_curve_type(hist_curve,GTK_CURVE_TYPE_FREE);
}

void hist_calc_area(GdkImage *image)
{
  guint i,j,k,ix,iy,iz;
  int	Max,Min;
  unsigned long ofset,ofset2;
  guchar *img_mat = NULL;
  gushort fukasa;
  unsigned char cdat;
  short bpl,bpp;
  guchar *buf = NULL;
  gshort *img_p16=NULL;
  guchar *img_p8=NULL;
  gint *img_p32=NULL;
  gushort width,height;

  width = image->width;
  height = image->height;
  buf = (guchar *)realloc(buf,width*height);
  if(image->bpp<=3){ // For Win32
    bpp = image->bpp*8;
    if(image->bpp==3) bpl = image->bpl;
    else{
      bpl = image->bpl/(image->bpp);
    }
  }
  else{
    bpp = image->bpp*8;
    bpl = image->bpl/(image->bpp);
  }
  if(bpp==32){
    img_p32 = (gint *)image->mem;
  }
  if(bpp==16){
    img_p16 = (gshort *)image->mem;
  }
  if(bpp==8){
    img_p8 = (guchar *)image->mem;
  }
  for(i=0;i<height;i++){
    for(j=0;j<image->width;j++){
      if(bpp==32){
	buf[j + i*width] = img_p32[j + i*bpl];
      }
      if(bpp==16){
	buf[j + i*width] = img_p16[j + i*bpl];
      }
      if(bpp==8){
	buf[j + i*width] = img_p8[j + i*bpl];
      }
    }
  }

  memset(hist,0,sizeof(hist));
  Max = 0;
  Min = 255;

  if(seq_flag==TRANS){
    img_mat = (unsigned char *)realloc(img_mat,(unsigned)xdim*ydim);
    fukasa = ydim;
  }
  else{
    if(seq_flag==CORO){
      img_mat = (unsigned char *)realloc(img_mat,(unsigned)xdim*zdim);
      fukasa = zdim;
    }
    else{
      img_mat = (unsigned char *)realloc(img_mat,(unsigned)ydim*zdim);
      fukasa = zdim;
    }
  }

  img_mat = readplane(img,xdim,ydim,zdim,image_cell[curr_img_roi]->img_no);

  iz = (int)((float)width/zoom_factor);
  for(i=0,k=0;i<height;i++){
    for(j=0;j<width;j++){
      ofset = j + i*width;
      if(buf[ofset]==0) continue;
      ix = (int)((float)j/zoom_factor);
      iy = (int)((float)i/zoom_factor);
      if(revy_flag==1) ofset2 = ix + (fukasa - 1 - iy)*iz;
      else ofset2 = ix + iy*iz;
      cdat = img_mat[ofset2];
      if(cdat==0||cdat==255) continue;
      hist[cdat]++;
      k++;
    }
  }
  if(k==0) return;
}

void hist_calc()
{
  GdkImage *roi_image;

  roi_pixmap = gdk_pixmap_new(gpetview->window,image_cell[curr_img_roi]->r_width,image_cell[curr_img_roi]->r_height,-1);
  gdk_draw_rectangle(roi_pixmap,clearGC,TRUE,0,0,image_cell[curr_img_roi]->r_width,image_cell[curr_img_roi]->r_height);
  roi_draw(4,0,0);
  roi_image =  gdk_image_get(roi_pixmap,0,0,image_cell[curr_img_roi]->r_width,image_cell[curr_img_roi]->r_height);
  clear_rec();
  hist_calc_area(roi_image);
  gdk_pixmap_unref(roi_pixmap);
}

void hist_set_proc()
{
  if (roi_flag==PICKP) return; //no histogram
  hist_on_flag = 1;
  hist_calc();
  hist_out();
}

void on_histogram(GdkEventMotion *event)
{
  gshort width,height;
  gshort posx,posy;
  gfloat ratio_x,ratio_y;
  gchar message[50];

  width = hist_curve->graph.widget.allocation.width;
  height = hist_curve->graph.widget.allocation.height;
  posx = event->x;
  posy = height - event->y;
  ratio_x = (float)posx/(float)width;
  ratio_y = (float)posy/(float)height;
  ratio_x *= (float)hist_imax;
  ratio_y *= (float)hist_ymax;
  sprintf(message,"%g %g",ratio_x,ratio_y);
  put_string(message,0);
}
