/* Copyright 2000-2011 Hiroshi Watabe*/
/* $Id: main.c,v d247099911b6 2011/02/04 00:20:16 watabe $ */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "gpetview.h"
#include "interface.h"
#include "support.h"

GtkWidget *gpetview;
GdkVisual *gvisual;
GdkPixmap *canvas_pixmap;
GdkRgbCmap *pet_cmap;
GdkGC *canvasGC;
GdkGC *clearGC;
GdkGC *roiGC;
GdkGC *numberGC;
GtkWidget *pet_canvas;
GtkWidget *pet_viewport;
GtkWidget *roi_info_window;
GtkWidget *roiwindow;
GtkWidget *hist_window;
GtkWidget *fileselection;
GtkWidget *import_window;
GtkWidget *profile_window;
GtkWidget *colorbar_window;
GtkWidget *aboutDialog;
GtkWidget *clear_dialog;
GtkWidget *overwrite_dialog;
GtkWidget *quit_dialog;
GtkWidget *warning_dialog;

PangoContext *canvas_context;
PangoLayout *canvas_layout;
PangoFontDescription *canvas_fontdesc;

/* global variables */
short canvas_width,canvas_height;
short view_width,view_height;
char timestamp[255];
char temproidir[1024]; /* ROI directory name for tempolary*/
short line_width = 1; /* line width for ROI */

int
main (int argc, char *argv[])
{
  int num_arg;

#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif

  gtk_set_locale ();
  gtk_init (&argc, &argv);

#ifdef WIN32
  add_pixmap_directory ("pixmaps");
#else
  add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");
#endif

  window_ready_flag = 0; /* window is ready for display or not */
  gdk_rgb_init();
  gvisual = gdk_rgb_get_visual();
  gcolormap = gdk_rgb_get_cmap();
  gtk_widget_set_default_colormap (gcolormap);
  gtk_widget_set_default_visual (gvisual);
  /*
   * The following code was added by Glade to create one of each component
   * (except popup menus), just so that you see something after building
   * the project. Delete any components that you don't want shown initially.
   */
  gpetview = create_gpetview ();
  aboutDialog = create_aboutDialog ();
  filewindow = create_filewindow ();
  fileselection = create_fileselection ();
  settingshell = create_settingshell ();
  roi_info_window = create_roi_info_window ();
  hist_window = create_hist_window ();
  hdr_info_window = create_hdr_info_window ();
  profile_window = create_profile_window ();
  import_window = create_import_window ();
  roiwindow = create_roiwindow ();
  overwrite_dialog = create_overwrite_dialog ();
  colorbar_window = create_colorbar_window ();
  pet_canvas = lookup_widget(gpetview,"canvas");
  pet_viewport = lookup_widget(gpetview,"viewport1");
  warning_dialog = create_warning_dialog();

  /* set window size and initiliazion*/
  canvas_width = 1000;
  canvas_height = 1000;
  view_width = 750;
  view_height = 550;
  /* read .gpetviewrc */
  init_gpetview();
  sprintf(timestamp,"%ld",(long)time(NULL));
  /* checking arguments */
  num_arg = checkarg(argc,argv);
  if(canvas_width<view_width){
    canvas_width = view_width;
  }
  if(canvas_height<view_height){
    canvas_height = view_height;
  }
  gtk_widget_set_usize (pet_canvas, canvas_width, canvas_height);
  gtk_widget_set_usize (pet_viewport, view_width, view_height);

  gtk_widget_show (gpetview);
  window_ready_flag = 1; /* window is ready for display or not */
  /* create pixmap for display PET image */
  canvasGC = gdk_gc_new(gpetview->window);
  clearGC = gdk_gc_new(gpetview->window);
  roiGC = gdk_gc_new(gpetview->window);
  gdk_gc_set_line_attributes(roiGC,line_width,GDK_LINE_SOLID,GDK_CAP_ROUND, GDK_JOIN_ROUND); /* set line width for ROI */
  gdk_gc_set_line_attributes(canvasGC,line_width,GDK_LINE_SOLID,GDK_CAP_ROUND, GDK_JOIN_ROUND); /* set line width for ROI */
  numberGC = gdk_gc_new(gpetview->window);
  pet_cmap = (GdkRgbCmap *)g_malloc(sizeof(GdkRgbCmap));
  /* memory allocation and initialize */
  img_seq = (guint *)g_malloc(MAX_NSEQ*sizeof(int));
  if (img_seq==NULL){
    fprintf(stderr,"cannot allocate memory for img_seq\n");
    return 0;
  }
  memset(img_seq,0,MAX_NSEQ*sizeof(guint));

  canvas_pixmap = gdk_pixmap_new(gpetview->window,canvas_width,canvas_height,-1);
  if(canvas_pixmap==NULL){
    return 0;
  }
  /* Pango */
  canvas_context = gtk_widget_get_pango_context(pet_canvas);
  canvas_layout = pango_layout_new(canvas_context);
  canvas_fontdesc = pango_font_description_new(); 
  pango_font_description_set_family(canvas_fontdesc,"monospace"); 
  pango_layout_set_font_description(canvas_layout,canvas_fontdesc); 

  gdk_window_clear(pet_canvas->window);
  gdk_draw_rectangle(canvas_pixmap,clearGC,TRUE,0,0,canvas_width,canvas_height);
  gdk_draw_rectangle(pet_canvas->window,clearGC,TRUE,0,0,canvas_width,canvas_height);
 if(argc>1){/* assume last argument is image file */
    if(pipe_flag==0&&num_arg!=0){
      fname = (char *)malloc(strlen(argv[argc - 1]) + 1);
      if(fname==NULL){
	fprintf(stderr,"Cannot alloc memory\n");
	exit(1);
      }
      strcpy(fname,argv[argc - 1]);
      set_filename();
    }
    draw_colorbar();
    view_proc(1);
  }
  gtk_main ();
  return 0;
}

