/* Copyright 2000 - 2011 Hiroshi Watabe*/
/* masking ROI */
/* $Id: roi_mask.c,v ad5996c60c95 2011/02/03 09:07:37 watabe $ */
#include "gpetview.h"

int ext_area(GdkImage*);

void roi_mask()
{
  gushort x,y,w,h;
  GdkImage *roi_image;
  char string[1024];
  FILE *fp;
  ROIINFO roi_info;

  if(roi_draw_flag==0){return;}

  roi_num++;
  x = image_cell[curr_img_roi]->r_left;
  y = image_cell[curr_img_roi]->r_top;
  w = image_cell[curr_img_roi]->r_width;
  h = image_cell[curr_img_roi]->r_height;

  roi_pixmap = gdk_pixmap_new(gpetview->window,image_cell[curr_img_roi]->r_width,image_cell[curr_img_roi]->r_height,-1);
  /* clear roi_pixmap */
  gdk_draw_rectangle(roi_pixmap,clearGC,TRUE,0,0,image_cell[curr_img_roi]->r_width,image_cell[curr_img_roi]->r_height);
  roi_draw(4,0,0);
  roi_image = gdk_image_get(roi_pixmap,0,0,image_cell[curr_img_roi]->r_width,image_cell[curr_img_roi]->r_height);
  ext_area(roi_image);
  roi_draw(5,0,0);
  redraw_canvas();
  gdk_pixmap_unref(roi_pixmap);

/* save temporary roi in /tmp */
#ifdef WIN32
  if (flag_roidir==1){
    sprintf(string,"%s\\roi%s.%d",temproidir,timestamp,roi_num);
  }
  else{
    sprintf(string,"roi%s.%d",timestamp,roi_num);
  }
#else
  if(flag_roidir==1){
    sprintf(string,"%s/roi%s.%d",temproidir,timestamp,roi_num);
  }
  else{
    sprintf(string,"/tmp/roi%s.%d",timestamp,roi_num);
  }
#endif
  fp = fopen(string,"wb");
  roi_info.img_no = image_cell[curr_img_roi]->img_no;
  roi_info.orient = seq_flag;
  roi_info.zoom = zoom_factor;
  roi_info.revy_flag = revy_flag;
  memcpy(&(roi_info.figure),&figure,sizeof(figure));
  fwrite(&roi_info,sizeof(roi_info),1,fp);
  fclose(fp);
}

/* extract image from ROI */
int ext_area(GdkImage *image)
{
  gushort width,height;
  guint j,i,ix,iy,iz;
  unsigned long ofset,ofset2;
  guchar *img_mat = NULL;
  gushort fukasa;
  short bpl,bpp;
  guchar *buf = NULL;
  gshort *img_p16=NULL;
  guchar *img_p8=NULL;
  gint *img_p32=NULL;

  width = image->width;
  height = image->height;
  buf = (guchar *)g_realloc(buf,width*height);
  if(buf==NULL){
    fprintf(stderr,"cannot allocate memory for ROI\n");
    return(0);
  }
  memset(buf,0,width*height);/* clear buffer */
  if(image->bpp<=3){ // For Win32
    bpp = image->bpp*8;
    if(image->bpp==3) bpl = image->bpl;
    else{
      bpl = image->bpl/(image->bpp);
    }
  }
  else{
    bpp = image->bpp*8;
    bpl = image->bpl/(image->bpp);
  }

  if(bpp==32){
    img_p32 = (gint *)image->mem;
  }
  if(bpp==24){
    img_p8 = (guchar *)image->mem;
  }
  if(bpp==16){
    img_p16 = (gshort *)image->mem;
  }
  if(bpp==8){
    img_p8 = (guchar *)image->mem;
  }
  for(i=0;i<height;i++){
    for(j=0;j<image->width;j++){
      if(bpp==32){
	if(img_p32[j + i*bpl]){
	  buf[j + i*width] = 255;
	}
      }
      if(bpp==24){
	if(img_p8[j*3 + i*bpl]){
	  buf[j + i*width] = 255;
	}
      }
      if(bpp==16){
	if(img_p16[j + i*bpl]){
	  buf[j + i*width] = 255;
	}
      }
      if(bpp==8){
	if(img_p8[j + i*bpl]){
	  buf[j + i*width] = 255;
	}
      }
    }
  }

  if(seq_flag==TRANS){
    img_mat = (unsigned char *)realloc(img_mat,(unsigned)xdim*ydim);
    fukasa = ydim;
  }
  else{
    if(seq_flag==CORO){
      img_mat = (unsigned char *)realloc(img_mat,(unsigned)xdim*zdim);
      fukasa = zdim;
    }
    else{
      img_mat = (unsigned char *)realloc(img_mat,(unsigned)ydim*zdim);
      fukasa = zdim;
    }
  }

  img_mat = readplane(img,xdim,ydim,zdim,image_cell[curr_img_roi]->img_no);

  iz = (int)((float)width/zoom_factor);
  for(i=0;i<height;i++){
    for(j=0;j<width;j++){
      ofset = j + i*width;
      if(roi_mask_flag==ROI_MASK_IN){
	if(buf[ofset]!=0){
	  continue; /* don't do anything since this is inside the ROI */
	}
      }
      else{
	if(buf[ofset]==0){
	  continue; /* don't do anything since this is outside the ROI */
	}
      }
      ix = (int)((float)j/zoom_factor);
      iy = (int)((float)i/zoom_factor);
      if(revy_flag==1) ofset2 = ix + (fukasa - 1 - iy)*iz;
      else ofset2 = ix + iy*iz;
      img_mat[ofset2] = 0;
    }
  }
  writeplane(img,img_mat,xdim,ydim,zdim,image_cell[curr_img_roi]->img_no);
  g_free(buf);
  return 1;
}
